/* bluez.h generated by valac 0.56.18-dirty, the Vala compiler, do not modify */

#ifndef __BACKENDS_BLUEZ_BLUEZ_H__
#define __BACKENDS_BLUEZ_BLUEZ_H__

#include "folks/folks.h"
#include <glib-object.h>
#include <gio/gio.h>
#include <glib.h>
#include <stdlib.h>
#include <string.h>
#include <libebook-contacts/libebook-contacts.h>

G_BEGIN_DECLS

#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

#define FOLKS_BACKENDS_BLUE_Z_TYPE_BACKEND (folks_backends_blue_z_backend_get_type ())
#define FOLKS_BACKENDS_BLUE_Z_BACKEND(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FOLKS_BACKENDS_BLUE_Z_TYPE_BACKEND, FolksBackendsBlueZBackend))
#define FOLKS_BACKENDS_BLUE_Z_BACKEND_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FOLKS_BACKENDS_BLUE_Z_TYPE_BACKEND, FolksBackendsBlueZBackendClass))
#define FOLKS_BACKENDS_BLUE_Z_IS_BACKEND(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FOLKS_BACKENDS_BLUE_Z_TYPE_BACKEND))
#define FOLKS_BACKENDS_BLUE_Z_IS_BACKEND_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FOLKS_BACKENDS_BLUE_Z_TYPE_BACKEND))
#define FOLKS_BACKENDS_BLUE_Z_BACKEND_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FOLKS_BACKENDS_BLUE_Z_TYPE_BACKEND, FolksBackendsBlueZBackendClass))

typedef struct _FolksBackendsBlueZBackend FolksBackendsBlueZBackend;
typedef struct _FolksBackendsBlueZBackendClass FolksBackendsBlueZBackendClass;
typedef struct _FolksBackendsBlueZBackendPrivate FolksBackendsBlueZBackendPrivate;

#define FOLKS_BACKENDS_BLUE_Z_TYPE_PERSONA_STORE (folks_backends_blue_z_persona_store_get_type ())
#define FOLKS_BACKENDS_BLUE_Z_PERSONA_STORE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FOLKS_BACKENDS_BLUE_Z_TYPE_PERSONA_STORE, FolksBackendsBlueZPersonaStore))
#define FOLKS_BACKENDS_BLUE_Z_PERSONA_STORE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FOLKS_BACKENDS_BLUE_Z_TYPE_PERSONA_STORE, FolksBackendsBlueZPersonaStoreClass))
#define FOLKS_BACKENDS_BLUE_Z_IS_PERSONA_STORE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FOLKS_BACKENDS_BLUE_Z_TYPE_PERSONA_STORE))
#define FOLKS_BACKENDS_BLUE_Z_IS_PERSONA_STORE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FOLKS_BACKENDS_BLUE_Z_TYPE_PERSONA_STORE))
#define FOLKS_BACKENDS_BLUE_Z_PERSONA_STORE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FOLKS_BACKENDS_BLUE_Z_TYPE_PERSONA_STORE, FolksBackendsBlueZPersonaStoreClass))

typedef struct _FolksBackendsBlueZPersonaStore FolksBackendsBlueZPersonaStore;
typedef struct _FolksBackendsBlueZPersonaStoreClass FolksBackendsBlueZPersonaStoreClass;
typedef struct _FolksBackendsBlueZPersonaStorePrivate FolksBackendsBlueZPersonaStorePrivate;

#define ORG_BLUEZ_TYPE_DEVICE (org_bluez_device_get_type ())
#define ORG_BLUEZ_DEVICE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), ORG_BLUEZ_TYPE_DEVICE, orgbluezDevice))
#define ORG_BLUEZ_IS_DEVICE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), ORG_BLUEZ_TYPE_DEVICE))
#define ORG_BLUEZ_DEVICE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), ORG_BLUEZ_TYPE_DEVICE, orgbluezDeviceIface))

typedef struct _orgbluezDevice orgbluezDevice;
typedef struct _orgbluezDeviceIface orgbluezDeviceIface;

#define ORG_BLUEZ_TYPE_DEVICE_PROXY (org_bluez_device_proxy_get_type ())

#define ORG_BLUEZ_TYPE_ERROR (org_bluez_error_get_type ())

#define ORG_BLUEZ_OBEX_TYPE_CLIENT (org_bluez_obex_client_get_type ())
#define ORG_BLUEZ_OBEX_CLIENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), ORG_BLUEZ_OBEX_TYPE_CLIENT, orgbluezobexClient))
#define ORG_BLUEZ_OBEX_IS_CLIENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), ORG_BLUEZ_OBEX_TYPE_CLIENT))
#define ORG_BLUEZ_OBEX_CLIENT_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), ORG_BLUEZ_OBEX_TYPE_CLIENT, orgbluezobexClientIface))

typedef struct _orgbluezobexClient orgbluezobexClient;
typedef struct _orgbluezobexClientIface orgbluezobexClientIface;

#define ORG_BLUEZ_OBEX_TYPE_CLIENT_PROXY (org_bluez_obex_client_proxy_get_type ())

#define FOLKS_BACKENDS_BLUE_Z_TYPE_PERSONA (folks_backends_blue_z_persona_get_type ())
#define FOLKS_BACKENDS_BLUE_Z_PERSONA(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FOLKS_BACKENDS_BLUE_Z_TYPE_PERSONA, FolksBackendsBlueZPersona))
#define FOLKS_BACKENDS_BLUE_Z_PERSONA_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FOLKS_BACKENDS_BLUE_Z_TYPE_PERSONA, FolksBackendsBlueZPersonaClass))
#define FOLKS_BACKENDS_BLUE_Z_IS_PERSONA(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FOLKS_BACKENDS_BLUE_Z_TYPE_PERSONA))
#define FOLKS_BACKENDS_BLUE_Z_IS_PERSONA_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FOLKS_BACKENDS_BLUE_Z_TYPE_PERSONA))
#define FOLKS_BACKENDS_BLUE_Z_PERSONA_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FOLKS_BACKENDS_BLUE_Z_TYPE_PERSONA, FolksBackendsBlueZPersonaClass))

typedef struct _FolksBackendsBlueZPersona FolksBackendsBlueZPersona;
typedef struct _FolksBackendsBlueZPersonaClass FolksBackendsBlueZPersonaClass;
typedef struct _FolksBackendsBlueZPersonaPrivate FolksBackendsBlueZPersonaPrivate;

#define ORG_BLUEZ_OBEX_TYPE_PHONEBOOK_ACCESS (org_bluez_obex_phonebook_access_get_type ())
#define ORG_BLUEZ_OBEX_PHONEBOOK_ACCESS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), ORG_BLUEZ_OBEX_TYPE_PHONEBOOK_ACCESS, orgbluezobexPhonebookAccess))
#define ORG_BLUEZ_OBEX_IS_PHONEBOOK_ACCESS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), ORG_BLUEZ_OBEX_TYPE_PHONEBOOK_ACCESS))
#define ORG_BLUEZ_OBEX_PHONEBOOK_ACCESS_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), ORG_BLUEZ_OBEX_TYPE_PHONEBOOK_ACCESS, orgbluezobexPhonebookAccessIface))

typedef struct _orgbluezobexPhonebookAccess orgbluezobexPhonebookAccess;
typedef struct _orgbluezobexPhonebookAccessIface orgbluezobexPhonebookAccessIface;

#define ORG_BLUEZ_OBEX_TYPE_PHONEBOOK_ACCESS_PROXY (org_bluez_obex_phonebook_access_proxy_get_type ())

#define ORG_BLUEZ_OBEX_PHONEBOOK_ACCESS_TYPE_PHONEBOOK_ENTRY (org_bluez_obex_phonebook_access_phonebook_entry_get_type ())
typedef struct _orgbluezobexPhonebookAccessPhonebookEntry orgbluezobexPhonebookAccessPhonebookEntry;

#define ORG_BLUEZ_OBEX_PHONEBOOK_ACCESS_TYPE_PHONEBOOK_PULL (org_bluez_obex_phonebook_access_phonebook_pull_get_type ())
typedef struct _orgbluezobexPhonebookAccessPhonebookPull orgbluezobexPhonebookAccessPhonebookPull;

#define ORG_BLUEZ_OBEX_TYPE_TRANSFER (org_bluez_obex_transfer_get_type ())
#define ORG_BLUEZ_OBEX_TRANSFER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), ORG_BLUEZ_OBEX_TYPE_TRANSFER, orgbluezobexTransfer))
#define ORG_BLUEZ_OBEX_IS_TRANSFER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), ORG_BLUEZ_OBEX_TYPE_TRANSFER))
#define ORG_BLUEZ_OBEX_TRANSFER_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), ORG_BLUEZ_OBEX_TYPE_TRANSFER, orgbluezobexTransferIface))

typedef struct _orgbluezobexTransfer orgbluezobexTransfer;
typedef struct _orgbluezobexTransferIface orgbluezobexTransferIface;

#define ORG_BLUEZ_OBEX_TYPE_TRANSFER_PROXY (org_bluez_obex_transfer_proxy_get_type ())

struct _FolksBackendsBlueZBackend {
	FolksBackend parent_instance;
	FolksBackendsBlueZBackendPrivate * priv;
};

struct _FolksBackendsBlueZBackendClass {
	FolksBackendClass parent_class;
};

struct _FolksBackendsBlueZPersonaStore {
	FolksPersonaStore parent_instance;
	FolksBackendsBlueZPersonaStorePrivate * priv;
};

struct _FolksBackendsBlueZPersonaStoreClass {
	FolksPersonaStoreClass parent_class;
};

typedef enum  {
	ORG_BLUEZ_ERROR_NOT_READY,
	ORG_BLUEZ_ERROR_FAILED,
	ORG_BLUEZ_ERROR_IN_PROGRESS,
	ORG_BLUEZ_ERROR_ALREADY_CONNECTED,
	ORG_BLUEZ_ERROR_NOT_CONNECTED,
	ORG_BLUEZ_ERROR_DOES_NOT_EXIST,
	ORG_BLUEZ_ERROR_CONNECT_FAILED,
	ORG_BLUEZ_ERROR_NOT_SUPPORTED,
	ORG_BLUEZ_ERROR_INVALID_ARGUMENTS,
	ORG_BLUEZ_ERROR_AUTHENTICATION_CANCELED,
	ORG_BLUEZ_ERROR_AUTHENTICATION_FAILED,
	ORG_BLUEZ_ERROR_AUTHENTICATION_REJECTED,
	ORG_BLUEZ_ERROR_AUTHENTICATION_TIMEOUT,
	ORG_BLUEZ_ERROR_CONNECTION_ATTEMPT_FAILED
} orgbluezError;
#define ORG_BLUEZ_ERROR org_bluez_error_quark ()

struct _orgbluezDeviceIface {
	GTypeInterface parent_iface;
	void (*connect) (orgbluezDevice* self, GError** error);
	void (*disconnect) (orgbluezDevice* self, GError** error);
	void (*disconnect_profile) (orgbluezDevice* self, const gchar* uuid, GError** error);
	void (*pair) (orgbluezDevice* self, GError** error);
	void (*cancel_pairing) (orgbluezDevice* self, GError** error);
	gchar* (*get_address) (orgbluezDevice* self);
	gchar* (*get_name) (orgbluezDevice* self);
	gchar* (*get_icon) (orgbluezDevice* self);
	guint32 (*get_bluetooth_class) (orgbluezDevice* self);
	guint16 (*get_appearance) (orgbluezDevice* self);
	gchar** (*get_uuids) (orgbluezDevice* self, gint* result_length1);
	gboolean (*get_paired) (orgbluezDevice* self);
	gboolean (*get_connected) (orgbluezDevice* self);
	gboolean (*get_trusted) (orgbluezDevice* self);
	void (*set_trusted) (orgbluezDevice* self, gboolean value);
	gboolean (*get_blocked) (orgbluezDevice* self);
	void (*set_blocked) (orgbluezDevice* self, gboolean value);
	gchar* (*get_alias) (orgbluezDevice* self);
	void (*set_alias) (orgbluezDevice* self, const gchar* value);
	char* (*get_adapter) (orgbluezDevice* self);
	gboolean (*get_legacy_pairing) (orgbluezDevice* self);
	gchar* (*get_mod_alias) (orgbluezDevice* self);
	gint16 (*get_rssi) (orgbluezDevice* self);
};

struct _orgbluezobexClientIface {
	GTypeInterface parent_iface;
	void (*create_session) (orgbluezobexClient* self, const gchar* address, GHashTable* args, GAsyncReadyCallback _callback_, gpointer _user_data_);
	char* (*create_session_finish) (orgbluezobexClient* self, GAsyncResult* _res_, GError** error);
	void (*remove_session) (orgbluezobexClient* self, const char* session, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*remove_session_finish) (orgbluezobexClient* self, GAsyncResult* _res_, GError** error);
};

struct _FolksBackendsBlueZPersona {
	FolksPersona parent_instance;
	FolksBackendsBlueZPersonaPrivate * priv;
};

struct _FolksBackendsBlueZPersonaClass {
	FolksPersonaClass parent_class;
};

struct _orgbluezobexPhonebookAccessPhonebookEntry {
	gchar* vcard;
	gchar* name;
};

struct _orgbluezobexPhonebookAccessIface {
	GTypeInterface parent_iface;
	void (*select) (orgbluezobexPhonebookAccess* self, const gchar* location, const gchar* phonebook, GError** error);
	orgbluezobexPhonebookAccessPhonebookEntry* (*list) (orgbluezobexPhonebookAccess* self, GHashTable* filters, gint* result_length1, GError** error);
	gchar** (*list_filter_fields) (orgbluezobexPhonebookAccess* self, gint* result_length1, GError** error);
	void (*pull_all) (orgbluezobexPhonebookAccess* self, const gchar* target, GHashTable* filters, gchar** path, GHashTable** props, GError** error);
};

struct _orgbluezobexPhonebookAccessPhonebookPull {
	char* path;
	GHashTable* props;
};

struct _orgbluezobexTransferIface {
	GTypeInterface parent_iface;
	void (*cancel) (orgbluezobexTransfer* self, GError** error);
	gchar* (*get_status) (orgbluezobexTransfer* self);
	char* (*get_session) (orgbluezobexTransfer* self);
	gchar* (*get_name) (orgbluezobexTransfer* self);
	gchar* (*get_transfer_type) (orgbluezobexTransfer* self);
	gint64 (*get_time) (orgbluezobexTransfer* self);
	guint64 (*get_size) (orgbluezobexTransfer* self);
	guint64 (*get_transferred) (orgbluezobexTransfer* self);
	gchar* (*get_filename) (orgbluezobexTransfer* self);
};

VALA_EXTERN void module_init (FolksBackendStore* backend_store);
VALA_EXTERN void module_finalize (FolksBackendStore* backend_store);
VALA_EXTERN GType folks_backends_blue_z_backend_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FolksBackendsBlueZBackend, g_object_unref)
VALA_EXTERN FolksBackendsBlueZBackend* folks_backends_blue_z_backend_new (void);
VALA_EXTERN FolksBackendsBlueZBackend* folks_backends_blue_z_backend_construct (GType object_type);
VALA_EXTERN GType folks_backends_blue_z_persona_store_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FolksBackendsBlueZPersonaStore, g_object_unref)
VALA_EXTERN GType org_bluez_device_proxy_get_type (void) G_GNUC_CONST ;
VALA_EXTERN guint org_bluez_device_register_object (void* object,
                                        GDBusConnection* connection,
                                        const gchar* path,
                                        GError** error);
VALA_EXTERN GQuark org_bluez_error_quark (void);
VALA_EXTERN GType org_bluez_error_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType org_bluez_device_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (orgbluezDevice, g_object_unref)
VALA_EXTERN GType org_bluez_obex_client_proxy_get_type (void) G_GNUC_CONST ;
VALA_EXTERN guint org_bluez_obex_client_register_object (void* object,
                                             GDBusConnection* connection,
                                             const gchar* path,
                                             GError** error);
VALA_EXTERN GType org_bluez_obex_client_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (orgbluezobexClient, g_object_unref)
VALA_EXTERN FolksBackendsBlueZPersonaStore* folks_backends_blue_z_persona_store_new (orgbluezDevice* device,
                                                                         const gchar* object_path,
                                                                         orgbluezobexClient* obex_client);
VALA_EXTERN FolksBackendsBlueZPersonaStore* folks_backends_blue_z_persona_store_construct (GType object_type,
                                                                               orgbluezDevice* device,
                                                                               const gchar* object_path,
                                                                               orgbluezobexClient* obex_client);
VALA_EXTERN const gchar* folks_backends_blue_z_persona_store_get_display_name (FolksBackendsBlueZPersonaStore* self);
VALA_EXTERN const gchar* folks_backends_blue_z_persona_store_get_object_path (FolksBackendsBlueZPersonaStore* self);
VALA_EXTERN GType folks_backends_blue_z_persona_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FolksBackendsBlueZPersona, g_object_unref)
VALA_EXTERN FolksBackendsBlueZPersona* folks_backends_blue_z_persona_new (const gchar* vcard,
                                                              EVCard* card,
                                                              FolksPersonaStore* store,
                                                              gboolean is_user,
                                                              const gchar* iid);
VALA_EXTERN FolksBackendsBlueZPersona* folks_backends_blue_z_persona_construct (GType object_type,
                                                                    const gchar* vcard,
                                                                    EVCard* card,
                                                                    FolksPersonaStore* store,
                                                                    gboolean is_user,
                                                                    const gchar* iid);
VALA_EXTERN void org_bluez_obex_client_create_session (orgbluezobexClient* self,
                                           const gchar* address,
                                           GHashTable* args,
                                           GAsyncReadyCallback _callback_,
                                           gpointer _user_data_);
VALA_EXTERN char* org_bluez_obex_client_create_session_finish (orgbluezobexClient* self,
                                                   GAsyncResult* _res_,
                                                   GError** error);
VALA_EXTERN void org_bluez_obex_client_remove_session (orgbluezobexClient* self,
                                           const char* session,
                                           GAsyncReadyCallback _callback_,
                                           gpointer _user_data_);
VALA_EXTERN void org_bluez_obex_client_remove_session_finish (orgbluezobexClient* self,
                                                  GAsyncResult* _res_,
                                                  GError** error);
VALA_EXTERN GType org_bluez_obex_phonebook_access_proxy_get_type (void) G_GNUC_CONST ;
VALA_EXTERN guint org_bluez_obex_phonebook_access_register_object (void* object,
                                                       GDBusConnection* connection,
                                                       const gchar* path,
                                                       GError** error);
VALA_EXTERN GType org_bluez_obex_phonebook_access_phonebook_entry_get_type (void) G_GNUC_CONST ;
VALA_EXTERN orgbluezobexPhonebookAccessPhonebookEntry* org_bluez_obex_phonebook_access_phonebook_entry_dup (const orgbluezobexPhonebookAccessPhonebookEntry* self);
VALA_EXTERN void org_bluez_obex_phonebook_access_phonebook_entry_free (orgbluezobexPhonebookAccessPhonebookEntry* self);
VALA_EXTERN void org_bluez_obex_phonebook_access_phonebook_entry_copy (const orgbluezobexPhonebookAccessPhonebookEntry* self,
                                                           orgbluezobexPhonebookAccessPhonebookEntry* dest);
VALA_EXTERN void org_bluez_obex_phonebook_access_phonebook_entry_destroy (orgbluezobexPhonebookAccessPhonebookEntry* self);
G_DEFINE_AUTO_CLEANUP_CLEAR_FUNC (orgbluezobexPhonebookAccessPhonebookEntry, org_bluez_obex_phonebook_access_phonebook_entry_destroy)
VALA_EXTERN GType org_bluez_obex_phonebook_access_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (orgbluezobexPhonebookAccess, g_object_unref)
VALA_EXTERN void org_bluez_obex_phonebook_access_select (orgbluezobexPhonebookAccess* self,
                                             const gchar* location,
                                             const gchar* phonebook,
                                             GError** error);
VALA_EXTERN orgbluezobexPhonebookAccessPhonebookEntry* org_bluez_obex_phonebook_access_list (orgbluezobexPhonebookAccess* self,
                                                                                 GHashTable* filters,
                                                                                 gint* result_length1,
                                                                                 GError** error);
VALA_EXTERN gchar** org_bluez_obex_phonebook_access_list_filter_fields (orgbluezobexPhonebookAccess* self,
                                                            gint* result_length1,
                                                            GError** error);
VALA_EXTERN void org_bluez_obex_phonebook_access_pull_all (orgbluezobexPhonebookAccess* self,
                                               const gchar* target,
                                               GHashTable* filters,
                                               gchar** path,
                                               GHashTable** props,
                                               GError** error);
VALA_EXTERN GType org_bluez_obex_phonebook_access_phonebook_pull_get_type (void) G_GNUC_CONST ;
VALA_EXTERN orgbluezobexPhonebookAccessPhonebookPull* org_bluez_obex_phonebook_access_phonebook_pull_dup (const orgbluezobexPhonebookAccessPhonebookPull* self);
VALA_EXTERN void org_bluez_obex_phonebook_access_phonebook_pull_free (orgbluezobexPhonebookAccessPhonebookPull* self);
VALA_EXTERN void org_bluez_obex_phonebook_access_phonebook_pull_copy (const orgbluezobexPhonebookAccessPhonebookPull* self,
                                                          orgbluezobexPhonebookAccessPhonebookPull* dest);
VALA_EXTERN void org_bluez_obex_phonebook_access_phonebook_pull_destroy (orgbluezobexPhonebookAccessPhonebookPull* self);
G_DEFINE_AUTO_CLEANUP_CLEAR_FUNC (orgbluezobexPhonebookAccessPhonebookPull, org_bluez_obex_phonebook_access_phonebook_pull_destroy)
VALA_EXTERN GType org_bluez_obex_transfer_proxy_get_type (void) G_GNUC_CONST ;
VALA_EXTERN guint org_bluez_obex_transfer_register_object (void* object,
                                               GDBusConnection* connection,
                                               const gchar* path,
                                               GError** error);
VALA_EXTERN GType org_bluez_obex_transfer_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (orgbluezobexTransfer, g_object_unref)
VALA_EXTERN void org_bluez_obex_transfer_cancel (orgbluezobexTransfer* self,
                                     GError** error);
VALA_EXTERN gchar* org_bluez_obex_transfer_get_status (orgbluezobexTransfer* self);
VALA_EXTERN char* org_bluez_obex_transfer_get_session (orgbluezobexTransfer* self);
VALA_EXTERN gchar* org_bluez_obex_transfer_get_name (orgbluezobexTransfer* self);
VALA_EXTERN gchar* org_bluez_obex_transfer_get_transfer_type (orgbluezobexTransfer* self);
VALA_EXTERN gint64 org_bluez_obex_transfer_get_time (orgbluezobexTransfer* self);
VALA_EXTERN guint64 org_bluez_obex_transfer_get_size (orgbluezobexTransfer* self);
VALA_EXTERN guint64 org_bluez_obex_transfer_get_transferred (orgbluezobexTransfer* self);
VALA_EXTERN gchar* org_bluez_obex_transfer_get_filename (orgbluezobexTransfer* self);
VALA_EXTERN void org_bluez_device_connect (orgbluezDevice* self,
                               GError** error);
VALA_EXTERN void org_bluez_device_disconnect (orgbluezDevice* self,
                                  GError** error);
VALA_EXTERN void org_bluez_device_disconnect_profile (orgbluezDevice* self,
                                          const gchar* uuid,
                                          GError** error);
VALA_EXTERN void org_bluez_device_pair (orgbluezDevice* self,
                            GError** error);
VALA_EXTERN void org_bluez_device_cancel_pairing (orgbluezDevice* self,
                                      GError** error);
VALA_EXTERN gchar* org_bluez_device_get_address (orgbluezDevice* self);
VALA_EXTERN gchar* org_bluez_device_get_name (orgbluezDevice* self);
VALA_EXTERN gchar* org_bluez_device_get_icon (orgbluezDevice* self);
VALA_EXTERN guint32 org_bluez_device_get_bluetooth_class (orgbluezDevice* self);
VALA_EXTERN guint16 org_bluez_device_get_appearance (orgbluezDevice* self);
VALA_EXTERN gchar** org_bluez_device_get_uuids (orgbluezDevice* self,
                                    gint* result_length1);
VALA_EXTERN gboolean org_bluez_device_get_paired (orgbluezDevice* self);
VALA_EXTERN gboolean org_bluez_device_get_connected (orgbluezDevice* self);
VALA_EXTERN gboolean org_bluez_device_get_trusted (orgbluezDevice* self);
VALA_EXTERN void org_bluez_device_set_trusted (orgbluezDevice* self,
                                   gboolean value);
VALA_EXTERN gboolean org_bluez_device_get_blocked (orgbluezDevice* self);
VALA_EXTERN void org_bluez_device_set_blocked (orgbluezDevice* self,
                                   gboolean value);
VALA_EXTERN gchar* org_bluez_device_get_alias (orgbluezDevice* self);
VALA_EXTERN void org_bluez_device_set_alias (orgbluezDevice* self,
                                 const gchar* value);
VALA_EXTERN char* org_bluez_device_get_adapter (orgbluezDevice* self);
VALA_EXTERN gboolean org_bluez_device_get_legacy_pairing (orgbluezDevice* self);
VALA_EXTERN gchar* org_bluez_device_get_mod_alias (orgbluezDevice* self);
VALA_EXTERN gint16 org_bluez_device_get_rssi (orgbluezDevice* self);

G_END_DECLS

#endif
