/* types.c generated by valac 0.56.18-dirty, the Vala compiler
 * generated from types.vala, do not modify */

/*
 * Copyright (C) 2010 Collabora Ltd.
 *
 * This library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *       Travis Reitter <travis.reitter@collabora.co.uk>
 */

#include "folks/folks.h"
#include <glib.h>
#include <string.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

static inline gpointer _vala_memdup2 (gconstpointer mem,
                        gsize byte_size);

/**
   * A 'boolean' type that has a distinct 'unset' state.
   *
   * @since 0.3.1
   */
static GType
folks_maybe_bool_get_type_once (void)
{
	static const GEnumValue values[] = {{FOLKS_MAYBE_BOOL_UNSET, "FOLKS_MAYBE_BOOL_UNSET", "unset"}, {FOLKS_MAYBE_BOOL_FALSE, "FOLKS_MAYBE_BOOL_FALSE", "false"}, {FOLKS_MAYBE_BOOL_TRUE, "FOLKS_MAYBE_BOOL_TRUE", "true"}, {0, NULL, NULL}};
	GType folks_maybe_bool_type_id;
	folks_maybe_bool_type_id = g_enum_register_static ("FolksMaybeBool", values);
	return folks_maybe_bool_type_id;
}

GType
folks_maybe_bool_get_type (void)
{
	static volatile gsize folks_maybe_bool_type_id__once = 0;
	if (g_once_init_enter (&folks_maybe_bool_type_id__once)) {
		GType folks_maybe_bool_type_id;
		folks_maybe_bool_type_id = folks_maybe_bool_get_type_once ();
		g_once_init_leave (&folks_maybe_bool_type_id__once, folks_maybe_bool_type_id);
	}
	return folks_maybe_bool_type_id__once;
}

static inline gpointer
_vala_memdup2 (gconstpointer mem,
               gsize byte_size)
{
	gpointer new_mem;
	if (mem && byte_size != 0) {
		new_mem = g_malloc (byte_size);
		memcpy (new_mem, mem, byte_size);
	} else {
		new_mem = NULL;
	}
	return new_mem;
}

