/* folks-internal.h generated by valac 0.56.18-dirty, the Vala compiler, do not modify */

#ifndef __FOLKS_FOLKS_INTERNAL_H__
#define __FOLKS_FOLKS_INTERNAL_H__

#include <glib-object.h>
#include <gee.h>
#include <glib.h>
#include <stdlib.h>
#include <string.h>

G_BEGIN_DECLS

#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

typedef struct _FolksInternalProfileBlock FolksInternalProfileBlock;

struct _FolksInternalProfileBlock {
	gchar* name;
	gint64 start;
};

VALA_EXTERN gboolean folks_internal_equal_sets (GType g_type,
                                    GBoxedCopyFunc g_dup_func,
                                    GDestroyNotify g_destroy_func,
                                    GeeSet* a,
                                    GeeSet* b);
VALA_EXTERN void folks_internal_profiling_point (const gchar* format,
                                     ...)  G_GNUC_PRINTF(1,2) ;
VALA_EXTERN void folks_internal_profile_block_free (FolksInternalProfileBlock * self);
G_DEFINE_AUTOPTR_CLEANUP_FUNC (FolksInternalProfileBlock, folks_internal_profile_block_free)
VALA_EXTERN FolksInternalProfileBlock* folks_internal_profiling_start (const gchar* format,
                                                           ...);
VALA_EXTERN void folks_internal_profiling_end (FolksInternalProfileBlock* block);

G_END_DECLS

#endif
