/* bluez-backend-factory.c generated by valac 0.56.18-dirty, the Vala compiler
 * generated from bluez-backend-factory.vala, do not modify */

/*
 * Copyright (C) 2009 Zeeshan Ali (Khattak) <zeeshanak@gnome.org>.
 * Copyright (C) 2009 Nokia Corporation.
 * Copyright (C) 2012-2013 Collabora Ltd.
 *
 * This library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *          Arun Raghavan <arun.raghavan@collabora.co.uk>
 *
 * Based on kf-backend-factory.vala by:
 *          Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
 *          Travis Reitter <travis.reitter@collabora.co.uk>
 *          Philip Withnall <philip.withnall@collabora.co.uk>
 */

#include "bluez.h"
#include "folks/folks.h"
#include <glib.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

/**
 * The backend module entry point.
 *
 * @backend_store a store to add the BlueZ backends to
 * @since 0.9.6
 */
void
module_init (FolksBackendStore* backend_store)
{
	FolksBackendsBlueZBackend* _tmp0_;
	FolksBackendsBlueZBackend* _tmp1_;
	g_return_if_fail (backend_store != NULL);
	_tmp0_ = folks_backends_blue_z_backend_new ();
	_tmp1_ = _tmp0_;
	folks_backend_store_add_backend (backend_store, (FolksBackend*) _tmp1_);
	_g_object_unref0 (_tmp1_);
}

/**
 * The backend module exit point.
 *
 * @param backend_store the store to remove the backends from
 * @since 0.9.6
 */
void
module_finalize (FolksBackendStore* backend_store)
{
	g_return_if_fail (backend_store != NULL);
}

